import numpy as np
import networkx as nx
from scipy import linalg as LA

import matplotlib.pyplot as plt
from sklearn.neighbors import KDTree
import scipy.io as sio

from mayavi import mlab
import yang.myutil as myutil

def read_off(file):
    """
    Function reads '.off' file
    """
    if 'OFF' != file.readline().strip():
        raise ('Not a valid OFF header')
    n_verts, n_faces, n_dontknow = tuple([int(s) for s in file.readline().strip().split(' ')])
    verts = [[float(s) for s in file.readline().strip().split(' ')] for i_vert in range(n_verts)]
    faces = [[int(s) for s in file.readline().strip().split(' ')][1:] for i_face in range(n_faces)]
    verts = np.array(verts)
    faces = np.array(faces)
    return verts, faces


def load_obj(filename):
    vertices_ = []
    faces_ = []
    img_textures_ = []

    for line in open(filename):
        if line.startswith('#'):
            continue
        values = line.split()
        if values[0] == 'v':
            v = [float(x) for x in values[1:4]]
            v = [v[0], v[1], v[2]]
            vertices_.append(v)
        elif values[0] == 'vt':
            img_textures_.append([values[1], values[2]])
        elif values[0] == 'f':
            f = [int(values[1].split('/')[0])-1,
                 int(values[2].split('/')[0])-1,
                 int(values[3].split('/')[0])-1]
            faces_.append(f)
    return np.array(vertices_), np.array(faces_)

def write_off(filename, vertex, face):
    """
    Function write up vertex anf face info to '.off' file
    vertex
    """
    with open(filename, 'a') as f:
        f.write('OFF\n')
        f.write('%d %d 0\n' % (vertex.shape[0], face.shape[0]))
        np.savetxt(f, vertex, fmt='%f %f %f')
        np.savetxt(f, face, fmt='3 %d %d %d')


def deg_cal(in_faces):
    """
    Based on the input face info, aligned the format to be a graph format
    and read out the degree vector
    """
    # print("in_faces_type", in_faces[:3])
    out_faces = np.concatenate([in_faces[:, :2], in_faces[:, 1:3], in_faces[:, [0, 2]]], axis=0)
    out_faces = np.concatenate([out_faces, out_faces[:, ::-1]], axis=0)
    out_faces = np.unique(out_faces, axis=0)
    _, deg = np.unique(out_faces[:, 0], return_counts=True)
    return out_faces, deg


def simplifiedMapping(simp_vertex, org_vertex):
    """
    This function is able to map the simplified vertex
    to the orginal vertex by using k-d tree
    k = 1, d = 3
    """
    kd_tree = KDTree(simp_vertex)
    _, idx = kd_tree.query(org_vertex, k=1)
    return idx


def gaussian_smoothing_dynamic(vertex_info, adj_mat, alpha=1):
    """
    Gaussian mesh smoothing, the sigma value is totally based on the 1-ring neighbors
    `alpha` is denoted as `t` in the paper
    """
    new_ver = np.zeros(vertex_info.shape).astype(np.float64)
    for i in range(vertex_info.shape[0]):
        adj_set = np.argwhere(np.array(adj_mat[i, :]).reshape(-1) == 1)
        adj_coord = vertex_info[adj_set, :].reshape(-1, 3)
        ver_i_coord = vertex_info[i, :].reshape(-1, 3)
        weight_set = np.zeros(adj_set.shape[0]).astype(np.float)
        sigma = np.sum(np.sqrt(np.sum((ver_i_coord - adj_coord) ** 2, axis=1))) / adj_set.shape[0] * alpha
        for j in range(adj_set.shape[0]):
            d_temp = np.sum((ver_i_coord - adj_coord[j, :]) ** 2).astype(np.float64)
            weight_set[j] = (1 / np.sqrt(2 * np.pi * sigma ** 2)) * np.exp(-d_temp / (2 * (sigma ** 2)))
        weight_set = weight_set / np.sum(weight_set)
        new_ver[i, :] = np.dot(weight_set.T, adj_coord)
    return new_ver


def gaussian_smoothing(vertex_info, adj_mat, alpha=1):
    """
    Another version of gaussian mesh smoothing with a constant alpha vlaue as the sigma
    """
    new_ver = np.zeros(vertex_info.shape).astype(np.float64)
    for i in range(vertex_info.shape[0]):
        adj_set = np.argwhere(np.array(adj_mat[i, :]).reshape(-1) == 1)
        adj_coord = vertex_info[adj_set, :].reshape(-1, 3)
        ver_i_coord = vertex_info[i, :].reshape(-1, 3)
        weight_set = np.zeros(adj_set.shape[0]).astype(np.float64)
        sigma = alpha
        for j in range(adj_set.shape[0]):
            d_temp = np.sum((ver_i_coord - adj_coord[j, :]) ** 2).astype(np.float64)
            weight_set[j] = (1 / np.sqrt(2 * np.pi * sigma ** 2)) * np.exp(-d_temp / (2 * (sigma ** 2)))
        weight_set = weight_set / np.sum(weight_set)
        new_ver[i, :] = np.dot(weight_set.T, adj_coord)
    return new_ver


def append_eigen(eigen, N):
    if (N % 2) == 1:
        out_eign = np.concatenate([np.repeat(eigen[0], N / 2), eigen, np.repeat(eigen[-1], N / 2)])
    else:
        raise ("Please input an odd number")
    return out_eign


def Measure_adjMat(face):
    """
    Return the adjcency matrix based on the input face
    """
    graph_edg_idx, deg_info = deg_cal(face)
    G = nx.Graph()
    G.add_edges_from(graph_edg_idx)
    adj_mat = nx.adjacency_matrix(G,
                                  nodelist=range(G.number_of_nodes())).todense().astype(float)
    return adj_mat


def RoughMeasure_Saliency(vers, fas):
    """
    Compute the Saliency based on one scale of vertex info
    """
    adj_mat = Measure_adjMat(fas)
    adj_cp = adj_mat.copy()

    adj_idx = np.argwhere(adj_mat == 1)
    for idx in adj_idx:
        i, j = idx
        normy = (vers[i, :] - verts[j, :]) ** 2
        factor = np.sum(normy)
        adj_cp[i, j] = 1 / factor

    # Compute the Laplacian matrix
    Lap_mat = np.diagflat(np.sum(adj_cp, axis=1)) - adj_cp
    # This step will take  the most of time, compute in O(n^3)
    # Python numpy sometimes give totally different eign_vec as MATLAB
    eign_val, eigen_vec = LA.eigh(Lap_mat)
    Log_eigen = np.log(np.abs(eign_val))
    # N is defined by paper
    N = 9
    spec_irregu = np.abs(Log_eigen -
                         np.convolve(append_eigen(Log_eigen, N),
                                     np.ones((N,)) / N, mode='valid'))
    Saliency_mat = np.multiply(np.dot(np.dot(eigen_vec,
                                             np.diagflat(np.exp(spec_irregu))),
                                      eigen_vec.T), adj_cp)
    Saliency_vec = Saliency_mat.sum(axis=1).reshape(-1)
    return Saliency_vec


# -------------------------------------------------------------------------------- #
def main_function():
    s = 10
    dymG_mesh_lst = []
    nmG_mesh_lst = []
    adj_matt = Measure_adjMat(faces)
    nmG_mesh_lst.append(verts)
    dymG_mesh_lst.append(verts)
    for ii in range(s - 1):
        nmG_mesh_lst.append(gaussian_smoothing(verts, adj_matt, alpha=(ii + 1) * 2))
        dymG_mesh_lst.append(gaussian_smoothing_dynamic(verts, adj_matt, alpha=(ii + 1) * 2))

    Sal_mat = np.zeros((s, verts.shape[0])).astype(np.float)
    Sal_ori = np.zeros((s, ori_verts.shape[0])).astype(np.float)
    for idx in range(s):
        Sal_tmp = RoughMeasure_Saliency(nmG_mesh_lst[idx], faces)
        Sal_tmp_dym = RoughMeasure_Saliency(dymG_mesh_lst[idx], faces)
        Sal_hat = np.abs(Sal_tmp_dym - Sal_tmp)  # Get the S_hat_i
    Sal_mat[idx, :] = Sal_hat

    idx_mapping = simplifiedMapping(nmG_mesh_lst[idx], ori_verts)
    Sal_ori[idx, :] = Sal_hat[:, idx_mapping].reshape(-1)

    Sal_out = np.log(np.sum(Sal_ori, axis=0))

    # sio.savemat('./tst/sal.mat', {'sal': Sal_out})
    return Sal_out
# -------------------------------------------------------------------------------- #


# ---------------------------alternative algorithm-------------------------------- #
def alternative_algorithm():
    s = 10
    dymG_mesh_lst = []
    nmG_mesh_lst = []
    adj_matt = Measure_adjMat(faces)
    nmG_mesh_lst.append(verts)
    dymG_mesh_lst.append(verts)
    for ii in range(s - 1):
        nmG_mesh_lst.append(gaussian_smoothing(verts, adj_matt, alpha=(ii + 1) * 2))
        dymG_mesh_lst.append(gaussian_smoothing_dynamic(verts, adj_matt, alpha=(ii + 1) * 2))

    Sal_mat = np.zeros((s, ori_verts.shape[0])).astype(np.float)
    Sal_ori = np.zeros((s - 1, ori_verts.shape[0])).astype(np.float)
    for idx in range(s):
        if idx == 0:
            Sal_hat = RoughMeasure_Saliency(nmG_mesh_lst[idx], faces)
        else:
            Sal_tmp_dym = RoughMeasure_Saliency(dymG_mesh_lst[idx], faces)
            Sal_hat = Sal_tmp_dym  # Get the S_hat_i

        idx_mapping = simplifiedMapping(nmG_mesh_lst[idx], ori_verts)
        Sal_mat[idx, :] = Sal_hat[:, idx_mapping].reshape(-1)

        if idx != 0:
            Sal_ori[idx - 1, :] = np.abs(Sal_mat[idx, :] - Sal_mat[idx - 1, :])

    Sal_out = np.log(np.sum(Sal_ori, axis=0))
    sio.savemat('./tst/sal2.mat', {'sal2': Sal_out})

# -------------------------------------------------------------------------------- #
# ---------------------------main function of program----------------------------- #
# Read the orginal mesh, which includes 15000 vertices
# with open('../data/0.off', 'r') as f:
#     ori_verts, ori_faces = read_off(f)
# # Read the pre-procesed QSlim simplified mesh, which only includes 1502 vertices, around 5000 faces
# with open('../data/simp.off', 'r') as f:
#     verts, faces = read_off(f)

ori_verts, ori_faces = load_obj('../data/bunny_simp_30_0.obj')
verts, faces = load_obj('../data/bunny_simp_10_0.obj')

# print(ori_verts[:3])
# print(ori_faces[:3])
# print(obj_ori_verts[:3])
# print(obj_ori_faces[:3])

# exit(0)

Sal_out = main_function()
print("Sal_out[:3]\n", Sal_out[:3])
print("Sal_out_len", len(Sal_out))
print("len_f, len_v", len(faces), len(verts))
print("len_f_ori, len_v_ori", len(ori_faces), len(ori_verts))
print(len(Sal_out))
# 输出的Sal-out表示的是每个点的显著度
# 频域分析的方法需要两个输入，一个是使用QSlim简化后的模型，一个是未简化的模型
# 然后通过这两个模型来进行模型显著度的计算
# -------------------------------------------------------------------------------- #

# -------------------------------------------------------------------------------- #
with open('../data/vertex_s.txt', 'w') as file:
    file.write(' '.join([str(i) for i in Sal_out]))
# -------------------------------------------------------------------------------- #

# ----------------------show the mesh--------------------------------------------- #
# mesh = trimesh.Trimesh(vertices=ori_verts, faces=ori_faces)
# count = 0
# for facet in range(len(ori_faces)):
#     tri_p = ori_faces[facet]
#     aver_value = np.mean([Sal_out[tri_p[0]], Sal_out[tri_p[1]], Sal_out[tri_p[2]]])
#     mesh.visual.face_colors[facet] = aver_value*255
# mesh.show()

fig = mlab.figure(size=(512, 512 + 48), bgcolor=(1, 1, 1))
face_value = []
for face_i in range(len(ori_faces)):
    tri_p = ori_faces[face_i]
    aver_value = np.mean([Sal_out[tri_p[0]], Sal_out[tri_p[1]], Sal_out[tri_p[2]]])
    face_value.append(aver_value)
myutil.mayavi_with_custom_face(ori_verts, ori_faces, face_value)
mlab.show()

# ----------------------show the mesh--------------------------------------------- #

